import argparse
import re

class PostProcessor:
	def __init__(self, filename):
		self.filename = filename

	def load(self):
		with open(self.filename, 'r') as file:
  			self.filedata = file.read()

	def save(self):
		with open(self.filename, 'w') as file:
		  file.write(self.filedata)

	# see https://www.overleaf.com/learn/latex/Questions/How_do_I_create_a_possessive_or_genitive_citation%3F
	# I want to be able to use "blabla @sedelmaier2014software's X is good" which will become "blabla \citeauthor{sedelmaier2014software}'s \citeyearpar{sedelmaier2014software} is good"
	def fix_inline_posessive_citets(self):
		self.filedata = re.sub(r"\\citet{(\w+)}'s", r"\\citeauthor{\1}'s \\citeyearpar{\1}", self.filedata)

	# a minimalized version of https://github.com/tomduck/pandoc-fignos/blob/master/pandoc_fignos.py
	# ![#label my caption](fig.jpg) becomes \caption{\#fig:slrrelation and \citet{fig:slrrelation}
	def fix_figure_and_table_labels(self):
		self.filedata = re.sub(r"\\caption(\[.+\])?{\\?\#(\w+):(\S+)\s", r"\\caption\1{\\label{\2:\3} ", self.filedata)

	# a minimalized version of https://github.com/tomduck/pandoc-secnos and others
	def fix_references_as_citations(self):
		self.filedata = re.sub(r"\\citet{fig:(\S+?)}", r"Figure~\\ref{fig:\1}", self.filedata)
		self.filedata = re.sub(r"\\citet{(pt\d+)}", r"Part~\\ref{\1}", self.filedata)
		self.filedata = re.sub(r"\\citet{((pt\d+)?ch\w+-\w+)}", r"Chapter~\\ref{\1}", self.filedata)
		self.filedata = re.sub(r"\\citet{tbl:(\S+?)}", r"Table~\\ref{tbl:\1}", self.filedata)
		self.filedata = re.sub(r"\\citet{sec:(\S+?)}", r"Section~\\ref{sec:\1}", self.filedata)

	# see https://github.com/Tufte-LaTeX/tufte-latex/issues/144
	def force_image_floats(self):
		fixfloat = r"\\checkoddpage \\ifoddpage \\forcerectofloat \\else \\forceversofloat \\fi"
		self.filedata = re.sub(r"\\begin{figure(\*?)}(\[?.*\]?)", r"\\begin{figure\1}\2\n" + fixfloat, self.filedata)

	# a marginfigure without [] is misaligned.
	def force_zero_offsest_on_margin_figures(self):
		self.filedata = re.sub(r"\\begin{marginfigure}\n", r"\\begin{marginfigure}[0mm]\n", self.filedata)


	# ![](bla.jpg){width=100%} results in \includegraphics[width=1\textwidth,height=\textheight]
	# I want it to be a \begin{figure*} for tufte-book instead of \begin{figure}
	def replace_full_width_image_with_image_star(self):
		lines = self.filedata.splitlines()
		linenrs = []
		opts = ["\\includegraphics[width=1.0\\textwidth]", "\\includegraphics[width=1\\textwidth,height=\\textheight]"]
		lines_to_scan_ahead = 15

		for l in enumerate(lines):
			for opt in opts:
				if opt in l[1]:
					linenrs.append(l[0])

		for l in linenrs:
			# 1. get rid of opts
			for opt in opts:
				lines[l] = lines[l].replace(opt, "\\includegraphics")
			# 2. go back and replace first \begin{figure} with \begin{figure*}
			for i in range(1, lines_to_scan_ahead):
				if l - i >= 0 and "\\begin{figure}[" in lines[l - i]:
					# already options present, most likely htp
					lines[l - i] = lines[l - i].replace("{figure}", "{figure*}")
					break
				if l - i >= 0 and "\\begin{figure}" in lines[l - i]:
					lines[l - i] = lines[l - i].replace("{figure}", "{figure*}[h!]")
					break
			# 3. go forward and replace first \end{figure} with \end{figure*}
			for i in range(1, lines_to_scan_ahead):
				if l + i < len(lines) and "\\end{figure}" in lines[l + i]:
					lines[l + i] = lines[l + i].replace("{figure}", "{figure*}")
					break

		self.filedata = "\n".join(lines)

	# inspired by https://kprussing.github.io/pandoc-acro/, but a lot simpler.
	def process_acronyms(self):
		self.filedata = re.sub(r"\s\+([A-Z]\w+)s", r" \\acfp{\1}", self.filedata)
		self.filedata = re.sub(r"\s\+([A-Z]\w+)", r" \\ac{\1}", self.filedata)

	def make_thoughts_of_first_words_of_piece(self, piece):
		# chapter starts with \chapter{Voorwoord}\label{voorwoord}} and blank lines
		# take the first words that do not start with \command{something} and wrap them in \newthought{}
		lines = self.filedata.splitlines()
		linenr = 0
		linenrs = []
		amount_of_words_to_wrap = 4

		for line in lines:
			if "\\" + piece + "{" in line or "\\" + piece + "[" in line:
				linenrs.append(linenr)
			linenr += 1
		for linenr_to_process in linenrs:
			linenr_of_first_words = linenr_to_process + 1

			found = False
			while not found:
				if len(lines[linenr_of_first_words]) < 2:
					linenr_of_first_words += 1
				elif lines[linenr_of_first_words].startswith("\\begin"):
					while not lines[linenr_of_first_words].startswith("\\end"):
						linenr_of_first_words += 1
					linenr_of_first_words += 1
				elif "\\label" in lines[linenr_of_first_words]:
					linenr_of_first_words += 1
				else:
					found = True

			words = lines[linenr_of_first_words].split()
			lines[linenr_of_first_words] = "\\newthought{" + " ".join(words[0:amount_of_words_to_wrap]) + "} " + " ".join(words[amount_of_words_to_wrap:])
		
		self.filedata = "\n".join(lines)

	def process(self):
		self.load()
		
		self.make_thoughts_of_first_words_of_piece('chapter')
		self.make_thoughts_of_first_words_of_piece('section')

		self.fix_inline_posessive_citets()
		self.fix_figure_and_table_labels()
		self.fix_references_as_citations()

		self.process_acronyms()

		self.replace_full_width_image_with_image_star()
		self.force_image_floats()
		self.force_zero_offsest_on_margin_figures()

		self.save()
		print('done, written to ' + self.filename)

parser = argparse.ArgumentParser(prog = 'Tufte Postprocessor', description = 'Tufte Postprocessor')
parser.add_argument('filename')
args = parser.parse_args()

processor = PostProcessor(args.filename)
processor.process()

